# Copyright (c) 2011 Canonical Ltd
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with this program. If not, see
# <http://www.gnu.org/licenses/>.

"""Things to ease working with cassandra."""

from pycassa.pool import InvalidRequestError


def workaround_1779(callable, *args, **kwargs):
    """Workaround cassandra not being able to do concurrent schema edits.

    The callable is tried until it does not raised InvalidRequestException
    with why = "Previous version mismatch. cannot apply."

    :param callable: The callable to call.
    :param args: The args for it.
    :param kwargs: The kwargs for it.
    :return: The result of calling the callable.
    """
    while True:
        # Workaround https://issues.apache.org/jira/browse/CASSANDRA-1779:
        # Cassandra cannot do concurrent schema changes.
        try:
            return callable(*args, **kwargs)
            break
        except InvalidRequestError as e:
            if e.why != 'Previous version mismatch. cannot apply.':
                raise
